//   Copyright (c) 2017-2019 Hartmut Kaiser
//   Copyright (c) 2018 Shahrzad Shirzad
//
//   Distributed under the Boost Software License, Version 1.0. (See accompanying
//   file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <phylanx/phylanx.hpp>

#include <hpx/hpx_main.hpp>
#include <hpx/include/lcos.hpp>
#include <hpx/testing.hpp>

#include <cstdint>
#include <iostream>
#include <string>
#include <utility>
#include <vector>

void test_and_operation_0d_false()
{
    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(false));

    phylanx::execution_tree::primitive and_operation;
    and_operation = phylanx::execution_tree::primitives::create_and_operation(
        hpx::find_here(),
        phylanx::execution_tree::primitive_arguments_type{
            std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(false),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d_double_false()
{
    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(2.0));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(0.0));

    phylanx::execution_tree::primitive and_operation;
    and_operation = phylanx::execution_tree::primitives::create_and_operation(
        hpx::find_here(),
        phylanx::execution_tree::primitive_arguments_type{
            std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(false),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d_true()
{
    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive and_operation;
    and_operation = phylanx::execution_tree::primitives::create_and_operation(
        hpx::find_here(),
        phylanx::execution_tree::primitive_arguments_type{
            std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(true),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d_double_true()
{
    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(2.0));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(1.0));

    phylanx::execution_tree::primitive and_operation;
    and_operation = phylanx::execution_tree::primitives::create_and_operation(
        hpx::find_here(),
        phylanx::execution_tree::primitive_arguments_type{
            std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(true),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d_lit_false()
{
    phylanx::ir::node_data<std::uint8_t> lhs(true);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(false));

    phylanx::execution_tree::primitive and_operation;
    and_operation = phylanx::execution_tree::primitives::create_and_operation(
        hpx::find_here(),
        phylanx::execution_tree::primitive_arguments_type{
            std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(false),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d_double_lit_false()
{
    phylanx::ir::node_data<double> lhs(2.0);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(0.0));

    phylanx::execution_tree::primitive and_operation;
    and_operation = phylanx::execution_tree::primitives::create_and_operation(
        hpx::find_here(),
        phylanx::execution_tree::primitive_arguments_type{
            std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(false),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d_lit_true()
{
    phylanx::ir::node_data<std::uint8_t> lhs(true);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive and_operation;
    and_operation = phylanx::execution_tree::primitives::create_and_operation(
        hpx::find_here(),
        phylanx::execution_tree::primitive_arguments_type{
            std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(true),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d_double_lit_true()
{
    phylanx::ir::node_data<double> lhs(2.0);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(1.0));

    phylanx::execution_tree::primitive and_operation;
    and_operation = phylanx::execution_tree::primitives::create_and_operation(
        hpx::find_here(),
        phylanx::execution_tree::primitive_arguments_type{
            std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(true),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d1d_false()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(false));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](bool x) { return (x && false); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d1d_lit_false()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(false);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](bool x) { return (x && false); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d1d_double_false()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(0.0));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<double> expected(v.size(), 0.0);

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d1d_double_lit_false()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<double> lhs(0.0);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<double> expected(v.size(), 0.0);

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d1d_true()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](bool x) { return (x && true); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d1d_lit_true()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(true);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](bool x) { return (x && true); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d1d_double_true()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(1.0));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](double x) { return (x != 0.0); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d1d_double_lit_true()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<double> lhs(1.0);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](double x) { return (x != 0.0); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d2d_false()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(false));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](bool x) { return (x && false); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d2d_lit_false()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(false);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](bool x) { return (x && false); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d2d_double_false()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(0.0));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<double> expected(m.rows(), m.columns(), 0.0);

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d2d_double_lit_false()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::ir::node_data<double> lhs(0.0);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<double> expected(m.rows(), m.columns(), 0.0);

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d2d_true()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](bool x) { return (x && true); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d2d_lit_true()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(true);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](bool x) { return (x && true); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d2d_double_true()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(2.0));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](double x) { return (x != 0); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_0d2d_double_lit_true()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::ir::node_data<double> lhs(2.0);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](double x) { return (x != 0); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v1 = gen.generate(100UL, 0, 1);
    blaze::DynamicVector<std::uint8_t> v2 = gen.generate(100UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v1));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v2));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v1, v2, [](bool x, bool y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d_lit()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v1 = gen.generate(1007UL, 0, 1);
    blaze::DynamicVector<std::uint8_t> v2 = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(v1);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v2));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v1, v2, [](bool x, bool y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d_double()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v1 = gen.generate(100UL, 0, 1);
    blaze::DynamicVector<double> v2 = gen.generate(100UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v1));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v2));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v1, v2, [](double x, double y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d_double_lit()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v1 = gen.generate(1007UL, 0, 1);
    blaze::DynamicVector<double> v2 = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<double> lhs(v1);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v2));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v1, v2, [](double x, double y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d0d_false()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(false));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](bool x) { return (x && false); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d0d_lit_false()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(v);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(false));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](bool x) { return (x && false); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d0d_double_false()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(0.0));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<double> expected(v.size(), 0.0);

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d0d_double_lit_false()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<double> lhs(v);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(0.0));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<double> expected(v.size(), 0.0);

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d0d_true()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](bool x) { return (x && true); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d0d_lit_true()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(v);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](bool x) { return (x && true); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d0d_double_true()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(2.0));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](double x) { return (x != 0.0); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d0d_double_lit_true()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(1007UL, 0, 1);

    phylanx::ir::node_data<double> lhs(v);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(2.0));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicVector<std::uint8_t> expected =
        blaze::map(v, [](double x) { return (x != 0.0); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d2d()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(104UL, 0, 1);

    blaze::Rand<blaze::DynamicMatrix<int>> mat_gen{};
    blaze::DynamicMatrix<std::uint8_t> m = mat_gen.generate(101UL, 104UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected{m.rows(), m.columns()};

    for (size_t i = 0UL; i < m.rows(); i++)
        blaze::row(expected, i) = blaze::map(blaze::row(m, i),
            blaze::trans(v),
            [](bool x, bool y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d2d_lit()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(104UL, 0, 1);

    blaze::Rand<blaze::DynamicMatrix<int>> mat_gen{};
    blaze::DynamicMatrix<std::uint8_t> m = mat_gen.generate(101UL, 104UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(v);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected{m.rows(), m.columns()};

    for (size_t i = 0UL; i < m.rows(); i++)
        blaze::row(expected, i) = blaze::map(blaze::row(m, i),
            blaze::trans(v),
            [](bool x, bool y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d2d_double()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(104UL, 0, 1);

    blaze::Rand<blaze::DynamicMatrix<int>> mat_gen{};
    blaze::DynamicMatrix<double> m = mat_gen.generate(101UL, 104UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected{m.rows(), m.columns()};

    for (size_t i = 0UL; i < m.rows(); i++)
        blaze::row(expected, i) = blaze::map(blaze::row(m, i),
            blaze::trans(v),
            [](double x, double y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_1d2d_double_lit()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(104UL, 0, 1);

    blaze::Rand<blaze::DynamicMatrix<int>> mat_gen{};
    blaze::DynamicMatrix<double> m = mat_gen.generate(101UL, 104UL, 0, 1);

    phylanx::ir::node_data<double> lhs(v);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected{m.rows(), m.columns()};

    for (size_t i = 0UL; i < m.rows(); i++)
        blaze::row(expected, i) = blaze::map(blaze::row(m, i),
            blaze::trans(v),
            [](double x, double y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m1 = gen.generate(1007UL, 1007UL, 0, 1);
    blaze::DynamicMatrix<std::uint8_t> m2 = gen.generate(1007UL, 1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m1));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m2));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m1, m2, [](bool x, bool y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d_lit()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m1 = gen.generate(1007UL, 1007UL, 0, 1);
    blaze::DynamicMatrix<std::uint8_t> m2 = gen.generate(1007UL, 1007UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(m1);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m2));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m1, m2, [](bool x, bool y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d_double()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m1 = gen.generate(1007UL, 1007UL, 0, 1);
    blaze::DynamicMatrix<double> m2 = gen.generate(1007UL, 1007UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m1));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m2));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m1, m2, [](double x, double y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d_double_lit()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m1 = gen.generate(1007UL, 1007UL, 0, 1);
    blaze::DynamicMatrix<double> m2 = gen.generate(1007UL, 1007UL, 0, 1);

    phylanx::ir::node_data<double> lhs(m1);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m2));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m1, m2, [](double x, double y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d0d_true()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](bool x) { return (x && true); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d0d_lit_true()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(m);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(true));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](bool x) { return (x && true); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d0d_false()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(false));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](bool x) { return (x && false); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d0d_lit_false()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<std::uint8_t> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(m);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(false));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](bool x) { return (x && false); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d0d_double_true()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(2.0));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](double x) { return (x != 0.0); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d0d_double_lit_true()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::ir::node_data<double> lhs(m);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(2.0));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected =
        blaze::map(m, [](double x) { return (x != 0.0); });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d0d_double_false()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(0.0));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<double> expected(m.rows(), m.columns(), 0.0);

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d0d_double_lit_false()
{
    blaze::Rand<blaze::DynamicMatrix<int>> gen{};
    blaze::DynamicMatrix<double> m = gen.generate(101UL, 101UL, 0, 1);

    phylanx::ir::node_data<double> lhs(m);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(0.0));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<double> expected(m.rows(), m.columns(), 0.0);

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d1d()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(104UL, 0, 1);

    blaze::Rand<blaze::DynamicMatrix<int>> mat_gen{};
    blaze::DynamicMatrix<std::uint8_t> m = mat_gen.generate(101UL, 104UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(m));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected{m.rows(), m.columns()};
    for (size_t i = 0UL; i < m.rows(); i++)
        blaze::row(expected, i) = blaze::map(blaze::row(m, i),
            blaze::trans(v),
            [](bool x, bool y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d1d_lit()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<std::uint8_t> v = gen.generate(104UL, 0, 1);

    blaze::Rand<blaze::DynamicMatrix<int>> mat_gen{};
    blaze::DynamicMatrix<std::uint8_t> m = mat_gen.generate(101UL, 104UL, 0, 1);

    phylanx::ir::node_data<std::uint8_t> lhs(m);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<std::uint8_t>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected{m.rows(), m.columns()};
    for (size_t i = 0UL; i < m.rows(); i++)
        blaze::row(expected, i) = blaze::map(blaze::row(m, i),
            blaze::trans(v),
            [](bool x, bool y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d1d_double()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(104UL, 0, 1);

    blaze::Rand<blaze::DynamicMatrix<int>> mat_gen{};
    blaze::DynamicMatrix<double> m = mat_gen.generate(101UL, 104UL, 0, 1);

    phylanx::execution_tree::primitive lhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(m));

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected{m.rows(), m.columns()};
    for (size_t i = 0UL; i < m.rows(); i++)
        blaze::row(expected, i) = blaze::map(blaze::row(m, i),
            blaze::trans(v),
            [](double x, double y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

void test_and_operation_2d1d_double_lit()
{
    blaze::Rand<blaze::DynamicVector<int>> gen{};
    blaze::DynamicVector<double> v = gen.generate(104UL, 0, 1);

    blaze::Rand<blaze::DynamicMatrix<int>> mat_gen{};
    blaze::DynamicMatrix<double> m = mat_gen.generate(101UL, 104UL, 0, 1);

    phylanx::ir::node_data<double> lhs(m);

    phylanx::execution_tree::primitive rhs =
        phylanx::execution_tree::primitives::create_variable(
            hpx::find_here(), phylanx::ir::node_data<double>(v));

    phylanx::execution_tree::primitive and_operation =
        phylanx::execution_tree::primitives::create_and_operation(
            hpx::find_here(),
            phylanx::execution_tree::primitive_arguments_type{
                std::move(lhs), std::move(rhs)});

    phylanx::execution_tree::primitive_argument_type f =
        and_operation.eval().get();

    blaze::DynamicMatrix<std::uint8_t> expected{m.rows(), m.columns()};
    for (size_t i = 0UL; i < m.rows(); i++)
        blaze::row(expected, i) = blaze::map(blaze::row(m, i),
            blaze::trans(v),
            [](double x, double y) { return x && y; });

    HPX_TEST_EQ(phylanx::ir::node_data<std::uint8_t>(std::move(expected)),
        phylanx::execution_tree::extract_boolean_value_strict(f));
}

///////////////////////////////////////////////////////////////////////////////
phylanx::execution_tree::primitive_argument_type compile_and_run(
    std::string const& codestr)
{
    phylanx::execution_tree::compiler::function_list snippets;
    phylanx::execution_tree::compiler::environment env =
        phylanx::execution_tree::compiler::default_environment();

    auto const& code = phylanx::execution_tree::compile(codestr, snippets, env);
    return code.run().arg_;
}

void test_operation(std::string const& code, std::string const& expected_str)
{
    HPX_TEST_EQ(compile_and_run(code), compile_and_run(expected_str));
}

///////////////////////////////////////////////////////////////////////////////
void test_operation_3d()
{
    // 0d
    test_operation(
        R"(true && astype([[[0, 0], [0, 1]], [[0, 0], [0, 0]]], "bool"))",
        R"(astype([[[0, 0], [0, 1]], [[0, 0], [0, 0]]], "bool"))");
    test_operation(
        R"(false && astype([[[0, 0], [0, 1]], [[0, 0], [0, 0]]], "bool"))",
        R"(astype([[[0, 0], [0, 0]], [[0, 0], [0, 0]]], "bool"))");
    test_operation(
        R"(astype([[[0, 0], [0, 1]], [[0, 0], [0, 0]]], "bool") && true)",
        R"(astype([[[0, 0], [0, 1]], [[0, 0], [0, 0]]], "bool"))");
    test_operation(
        R"(astype([[[0, 0], [0, 1]], [[0, 0], [0, 0]]], "bool") && false)",
        R"(astype([[[0, 0], [0, 0]], [[0, 0], [0, 0]]], "bool"))");

    // 1d
    test_operation(
        R"(astype([1, 1], "bool") &&
           astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))",
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))");
    test_operation(
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool") &&
           astype([1, 1], "bool") )",
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))");

    // 2d
    test_operation(
        R"(astype([[1, 1], [1, 1]], "bool") &&
           astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))",
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))");
    test_operation(
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool") &&
           astype([[1, 1], [1, 1]], "bool"))",
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))");

    // 3d
    test_operation(
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool") &&
           astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))",
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))");
    test_operation(
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool") &&
           astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))",
        R"(astype([[[0, 0], [1, 1]], [[0, 0], [0, 0]]], "bool"))");
}

void test_operation_4d()
{
    // 0d
    test_operation(
        R"(true && astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool"))",
        R"(astype([[[[0, 0, 1],[0, 1, 0]],[[0, 0, 0],[1, 0, 0]]],
                   [[[0, 1, 1],[1, 1, 0]],[[0, 0, 0],[1, 0, 0]]]], "bool"))");
    test_operation(
        R"(false && astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool"))",
        R"(astype([[[[0, 0, 0],[0, 0, 0]],[[0, 0, 0],[0, 0, 0]]],
                   [[[0, 0, 0],[0, 0, 0]],[[0, 0, 0],[0, 0, 0]]]], "bool"))");
    test_operation(
        R"(astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool") && true)",
        R"(astype([[[[0, 0, 1],[0, 1, 0]],[[0, 0, 0],[1, 0, 0]]],
                   [[[0, 1, 1],[1, 1, 0]],[[0, 0, 0],[1, 0, 0]]]], "bool"))");
    test_operation(
        R"(astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
           [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool") && false)",
        R"(astype([[[[0, 0, 0],[0, 0, 0]],[[0, 0, 0],[0, 0, 0]]],
                   [[[0, 0, 0],[0, 0, 0]],[[0, 0, 0],[0, 0, 0]]]], "bool"))");

    // 1d
    test_operation(
        R"([1, 0, 1] && astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool"))",
        R"(astype([[[[0, 0, 1],[0, 0, 0]],[[0, 0, 0],[1, 0, 0]]],
                   [[[0, 0, 1],[1, 0, 0]],[[0, 0, 0],[1, 0, 0]]]], "bool"))");
    test_operation(
        R"(astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool")
            && [1, 0, 1])",
        R"(astype([[[[0, 0, 1],[0, 0, 0]],[[0, 0, 0],[1, 0, 0]]],
                   [[[0, 0, 1],[1, 0, 0]],[[0, 0, 0],[1, 0, 0]]]], "bool"))");

    // 2d
    test_operation(
        R"([[1],[0]] && astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool"))",
        R"(astype([[[[0, 0, 1],[0, 0, 0]],[[0, 0, 0],[0, 0, 0]]],
                   [[[0, 1, 1],[0, 0, 0]],[[0, 0, 0],[0, 0, 0]]]], "bool"))");
    test_operation(
        R"(astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool")
            && [[1, 0, 0],[0, 1, 0]])",
        R"(astype([[[[0, 0, 0],[0, 1, 0]],[[0, 0, 0],[0, 0, 0]]],
                   [[[0, 0, 0],[0, 1, 0]],[[0, 0, 0],[0, 0, 0]]]], "bool"))");

    // 3d
    test_operation(
        R"([[[1, 0, 0]],[[0, 1, 0]]] &&
            astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool"))",
        R"(astype([[[[0, 0, 0],[0, 0, 0]],[[0, 0, 0],[0, 0, 0]]],
                   [[[0, 0, 0],[1, 0, 0]],[[0, 0, 0],[0, 0, 0]]]], "bool"))");
    test_operation(
        R"(astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool")
            && [[[1, 0, 0],[0, 1, 0]]])",
        R"(astype([[[[0, 0, 0],[0, 1, 0]],[[0, 0, 0],[0, 0, 0]]],
                   [[[0, 0, 0],[0, 1, 0]],[[0, 0, 0],[0, 0, 0]]]], "bool"))");

    // 4d
    test_operation(
        R"([[[[1],[0]],[[0],[1]]],[[[1],[0]],[[1],[0]]]] &&
            astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool"))",
        R"(astype([[[[0, 0, 1],[0, 0, 0]],[[0, 0, 0],[1, 0, 0]]],
                   [[[0, 1, 1],[0, 0, 0]],[[0, 0, 0],[0, 0, 0]]]], "bool"))");
    test_operation(
        R"([[[[1, 0, 0]],[[0, 1, 0]]],[[[1, 1, 1]],[[0, 1, 0]]]] &&
            astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool"))",
        R"(astype([[[[0, 0, 0],[0, 0, 0]],[[0, 0, 0],[0, 0, 0]]],
                   [[[0, 1, 1],[1, 1, 0]],[[0, 0, 0],[0, 0, 0]]]], "bool"))");
    test_operation(
        R"([[[[1, 0, 0],[1, 0, 0]]],[[[1, 0, 0],[0, 1, 0]]]] &&
            astype([[[[0, 0, 1], [0, 1, 0]], [[0, 0, 0], [1, 0, 0]]],
            [[[0, 1, 1], [1, 1, 0]], [[0, 0, 0], [1, 0, 0]]]], "bool"))",
        R"(astype([[[[0, 0, 0],[0, 0, 0]],[[0, 0, 0],[1, 0, 0]]],
                   [[[0, 0, 0],[0, 1, 0]],[[0, 0, 0],[0, 0, 0]]]], "bool"))");
}

///////////////////////////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
    test_and_operation_0d_false();
    test_and_operation_0d_true();
    test_and_operation_0d_lit_false();
    test_and_operation_0d_lit_true();
    test_and_operation_0d1d_false();
    test_and_operation_0d1d_lit_false();
    test_and_operation_0d1d_true();
    test_and_operation_0d1d_lit_true();
    test_and_operation_0d2d_true();
    test_and_operation_0d2d_lit_true();
    test_and_operation_0d2d_false();
    test_and_operation_0d2d_lit_false();

    test_and_operation_0d_double_false();
    test_and_operation_0d_double_true();
    test_and_operation_0d_double_lit_false();
    test_and_operation_0d_double_lit_true();
    test_and_operation_0d1d_double_false();
    test_and_operation_0d1d_double_lit_false();
    test_and_operation_0d1d_double_true();
    test_and_operation_0d1d_double_lit_true();
    test_and_operation_0d2d_double_true();
    test_and_operation_0d2d_double_lit_true();
    test_and_operation_0d2d_double_false();
    test_and_operation_0d2d_double_lit_false();

    test_and_operation_1d();
    test_and_operation_1d_lit();
    test_and_operation_1d0d_false();
    test_and_operation_1d0d_lit_false();
    test_and_operation_1d0d_true();
    test_and_operation_1d0d_lit_true();
    test_and_operation_1d2d();
    test_and_operation_1d2d_lit();

    test_and_operation_1d_double();
    test_and_operation_1d_double_lit();
    test_and_operation_1d0d_double_false();
    test_and_operation_1d0d_double_lit_false();
    test_and_operation_1d0d_double_true();
    test_and_operation_1d0d_double_lit_true();
    test_and_operation_1d2d_double();
    test_and_operation_1d2d_double_lit();

    test_and_operation_2d();
    test_and_operation_2d_lit();
    test_and_operation_2d0d_true();
    test_and_operation_2d0d_lit_true();
    test_and_operation_2d0d_false();
    test_and_operation_2d0d_lit_false();
    test_and_operation_2d1d();
    test_and_operation_2d1d_lit();

    test_and_operation_2d_double();
    test_and_operation_2d_double_lit();
    test_and_operation_2d0d_double_true();
    test_and_operation_2d0d_double_lit_true();
    test_and_operation_2d0d_double_false();
    test_and_operation_2d0d_double_lit_false();
    test_and_operation_2d1d_double();
    test_and_operation_2d1d_double_lit();

    test_operation("true && false", "false");
    test_operation("__and(true, false)", "false");
    test_operation("logical_and(true, false)", "false");

    test_operation_3d();

    test_operation_4d();

    return hpx::util::report_errors();
}
